from hyperliquid.api import API
from hyperliquid.utils.types import Interval, Json


class Info(API):
    def __init__(self):
        super().__init__()

    def user_data(self, address: str) -> Json:
        """Retrieve trading details about a user.

        POST /info

        Args:
            address (str): Onchain address in 42-character hexadecimal format;
                            e.g. 0x0000000000000000000000000000000000000000.

        Returns:
            {
              userState: UserState,
              openOrders: [FrontendOrder, ...],
              fills: [Fill, ...],
              ledgerUpdates: [LedgerUpdate, ...],
              totalPoolEquity: float string,
              agentAddress: str,
              pendingWithdrawals: [WithdrawalVoucher, ...],
              meta: Meta,
              assetCtxs: [AssetCtx, ...],
              serverTime: int,
              cumLedger: float string,
              leadingPools: [LeadingPool, ...],
              isPool: bool,
              user: str
            }

            The specific contents of these values (e.g. LeadingPool) are fleshed out
            in our Gitbook docs: https://hyperliquid.gitbook.io/hyperliquid-api-docs/
        """
        return self.post("/info", {"type": "webData", "user": address})

    def user_performance(self, address: str) -> Json:
        """Retrieve a user's trading performance.

        POST /info

        Args:
            address (str): Onchain address in 42-character hexadecimal format;
                            e.g. 0x0000000000000000000000000000000000000000.

        Returns:
            {
              day: WindowPortfolio,
              week: WindowPortfolio,
              month: WindowPortfolio
            }

            where WindowPortfolio is of type

            {
                accountValueHistory: [float, float],
                pnlHistory: [float, float],
                vlm: float string,
                totLedgerChange: float string,
                accountValueChange: PortfolioChange
            }
        """
        return self.post("/info", {"type": "portfolio", "user": address})

    def l2_book(self, coin: str) -> Json:
        """Retrieve the L2 orderbook data for a given coin.

        POST /info

        Args:
            coin (str): e.g. ETH, BTC, MATIC, ATOM

        Returns:
            {
              bids: [BookLevel, ...],
              asks: [BookLevel, ...]
            }

            where BookLevel is of type

            {
              px: str,
              sz: str,
              tot: str
            }
        """
        return self.post("/info", {"type": "l2Book", "coin": coin})

    def all_mids(self) -> Json:
        """Retrieve all mids for all actively traded coins.

        POST /info

        Returns:
            {
              ATOM: float string,
              BTC: float string,
              any other coins which are trading: float string
            }
        """
        return self.post("/info", {"type": "allMids"})

    def leaderboard(self) -> Json:
        """Retrieve the trader leaderboard.

        POST /info

        Returns:
            {
              leaderboardRows: [
                {
                  accountValue: float string,
                  ethAddress: str,
                  pastMonthVolume: float string,
                  prize: int,
                  rank: int
                },
                ...
              ]
            }
        """
        return self.post("/info", {"type": "leaderboard"})

    def user_fills(self, address: str) -> Json:
        """Retrieve a given user's fills.

        POST /info

        Args:
            address (str): Onchain address in 42-character hexadecimal format;
                            e.g. 0x0000000000000000000000000000000000000000.

        Returns:
            [
              {
                closedPnl: float string,
                coin: str,
                crossed: bool,
                dir: str,
                hash: str,
                oid: int,
                px: float string,
                side: str,
                startPosition: float string,
                sz: float string,
                time: int
              },
              ...
            ]
        """
        return self.post("/info", {"type": "userFills", "user": address})

    def user_pools(self, address: str) -> Json:
        """Retrieve pools which a given user is participating in.

        POST /info

        Args:
            address (str): Onchain address in 42-character hexadecimal format;
                            e.g. 0x0000000000000000000000000000000000000000.

        Returns:
            [
              {
                name: str,
                poolAddress: str,
                leader: str,
                description: str,
                portfolio: Portfolio,
                apr: float,
                followerState: Follower or None,
                ageInDays: int,
                leaderFraction: float,
                leaderCommission: float,
                roi: float,
                followers: [Follower, ...],
                maxDistributable: float,
                maxWithdrawable: float,
                isClosed: bool
              },
              ...
            ]

            where Follower is

            {
              user: str,
              poolEquity: float,
              pnl: float,
              allTimePnl: float,
              daysFollowing: int,
              poolEntryTime: int,
              lockupUntil: int
            }
        """
        return self.post("/info", {"type": "pools", "user": address})

    def pool_details(self, pool_address: str, user_address: str | None = None) -> Json:
        """Retrieve details about a specific pool.

        POST /info

        Args:
            pool_address (str): Onchain address in 42-character hexadecimal format;
                                e.g. 0x0000000000000000000000000000000000000000.
            user_address (str, optional): Onchain address in 42-character
                                          hexadecimal format; e.g.
                                          0x0000000000000000000000000000000000000000.

        Returns:
            {
              name: str,
              poolAddress: str,
              leader: str,
              description: str,
              portfolio: Portfolio,
              apr: float,
              followerState: Follower or None,
              ageInDays: int,
              leaderFraction: float,
              leaderCommission: float,
              roi: float,
              followers: [Follower, ...],
              maxDistributable: float,
              maxWithdrawable: float,
              isClosed: bool
            }

            where Follower is

            {
              user: str,
              poolEquity: float,
              pnl: float,
              allTimePnl: float,
              daysFollowing: int,
              poolEntryTime: int,
              lockupUntil: int
            }
        """
        if user_address is None:
            return self.post("/info", {"type": "poolDetails", "poolAddress": pool_address})
        else:
            return self.post(
                "/info",
                {
                    "type": "poolDetails",
                    "poolAddress": pool_address,
                    "user": user_address,
                },
            )

    def leading_pools(self, address: str) -> Json:
        """Retrieve pools which a given user created.

        POST /info

        Args:
            address (str): Onchain address in 42-character hexadecimal format;
                            e.g. 0x0000000000000000000000000000000000000000.

        Returns:
            [
              {
                address: str,
                name: str
              },
              ...
            ]
        """
        return self.post("/info", {"type": "leadingPools", "user": address})

    def candles(self, coin: str, interval: Interval, start_time: int, end_time: int = None) -> Json:
        """Retrieve OHLC candles.

        POST /info

        Args:
            coin (str): e.g. ETH, BTC, MATIC, ATOM
            interval (Interval): Interval enum type, 1min/15min/etc
            start_time (int): UNIX timestamp in millis for start time
            end_time (int, optional): UNIX timestamp in millis for end time

        Returns:
            [
              {
                T: int,
                o: float string,
                h: float string,
                l: float string,
                c: float string,
                i: str,
                n: int,
                s: str,
                t: int,
                v: float string
              },
              ...
            ]
        """
        if end_time is None:
            return self.post(
                "/info",
                {
                    "type": "candleSnapshot",
                    "req": {
                        "coin": coin,
                        "interval": interval.value,
                        "startTime": start_time,
                    },
                },
            )
        else:
            return self.post(
                "/info",
                {
                    "type": "candleSnapshot",
                    "req": {
                        "coin": coin,
                        "interval": interval.value,
                        "startTime": start_time,
                        "endTime": end_time,
                    },
                },
            )



import struct
import sys

from eth_abi import encode
from eth_account.messages import encode_structured_data
from eth_utils import keccak

ZERO_ADDRESS = "0x0000000000000000000000000000000000000000"


def order_type_to_array(order_type):
    if "limit" in order_type:
        tif = order_type["limit"]["tif"]
        if tif == "Gtc":
            return [2, 0]
        elif tif == "Alo":
            return [1, 0]
        elif tif == "Ioc":
            return [3, 0]
    elif "trigger" in order_type:
        trigger = order_type["trigger"]
        trigger_px = trigger["triggerPx"]
        if trigger["isMarket"] and trigger["tpsl"] == "tp":
            return [4, trigger_px]
        elif not trigger["isMarket"] and trigger["tpsl"] == "tp":
            return [5, trigger_px]
        elif trigger["isMarket"] and trigger["tpsl"] == "sl":
            return [6, trigger_px]
        elif not trigger["isMarket"] and trigger["tpsl"] == "sl":
            return [7, trigger_px]


def order_grouping_to_number(grouping):
    if grouping == "na":
        return 0
    elif grouping == "normalTpsl":
        return 1
    elif grouping == "positionTpsl":
        return 2


def double_to_byte_array(d):
    arr = bytearray(struct.pack("<d", d))
    if sys.byteorder == "little":
        arr.reverse()

    return arr


def order_spec_pre_processing(order_spec):
    decoded_order = order_spec["decodedOrder"]
    order_type_array = order_type_to_array(order_spec["orderType"])
    return (
        decoded_order["asset"],
        decoded_order["isBuyOrder"],
        double_to_byte_array(decoded_order["limitPx"]),
        double_to_byte_array(decoded_order["sz"]),
        decoded_order["isReduceOnly"],
        order_type_array[0],
        double_to_byte_array(order_type_array[1]),
    )


def construct_phantom_agent(signature_types, signature_data):
    connection_id = encode(signature_types, signature_data)

    return {"source": "a", "connectionId": keccak(connection_id)}


def sign_l1_action(wallet, signature_types, signature_data, active_pool, nonce):
    signature_types.append("address")
    signature_types.append("uint64")
    if active_pool is None:
        signature_data.append(ZERO_ADDRESS)
    else:
        signature_data.append(active_pool)
    signature_data.append(nonce)

    phantom_agent = construct_phantom_agent(signature_types, signature_data)

    data = {
        "domain": {
            "chainId": 1337,
            "name": "Exchange",
            "verifyingContract": "0x0000000000000000000000000000000000000000",
            "version": "1",
        },
        "types": {
            "Agent": [
                {"name": "source", "type": "string"},
                {"name": "connectionId", "type": "bytes32"},
            ],
            "EIP712Domain": [
                {"name": "name", "type": "string"},
                {"name": "version", "type": "string"},
                {"name": "chainId", "type": "uint256"},
            ],
        },
        "primaryType": "Agent",
        "message": phantom_agent,
    }

    structured_data = encode_structured_data(data)
    signed = wallet.sign_message(structured_data)
    return signed



from hyperliquid.api import API
from hyperliquid.utils.types import Json


class Explorer(API):
    def __init__(self):
        super().__init__()

    def block_details(self, height: int) -> Json:
        """Retrieve details about block.

        POST /explorer

        Args:
            height (int): The height of the block to retrieve.

        Returns:
            {
              type: "blockDetails",
              blockDetails: {
                height: int,
                blockTime: int,
                hash: str,
                proposer: str,
                numTxs: int,
                txs: [
                  {
                    time: int,
                    user: str,
                    action: {
                      type: str,
                      block: int,
                      hash: str
                      optional key(s) which depend on action type: *,
                    }
                  },
                  ...
                ]
              }
            }
        """
        return self.post("/explorer", {"type": "blockDetails", "height": height})

    def recent_block_list(self, n_blocks: int) -> Json:
        """Retrieve list of most recent n blocks.

        POST /explorer

        Args:
            n_blocks (int): How many blocks to retrieve.

        Returns:
            {
              type: "blockList",
              block_summaries: [
                {
                  height: int,
                  blockTime: int,
                  hash: str,
                  proposer: str,
                  numTxs: int
                },
                ...
              ]
            }
        """
        return self.post("/explorer", {"type": "blockList", "n_blocks": n_blocks})

    def tx_details(self, tx: str) -> Json:
        """Retrieve details about transaction.

        POST /explorer

        Args:
            tx (str): Hash of the transaction to retrieve.

        Returns:
            {
              type: "txDetails",
              tx: {
                time: int,
                user: str,
                action: {
                  type: str,
                  block: int,
                  hash: str
                  optional key(s) which depend on action type: *,
                }
              }
            }
        """
        return self.post("/explorer", {"type": "txDetails", "hash": tx})

    def user_details(self, address: str) -> Json:
        """Retrieve onchain details about a user.

        POST /explorer

        Args:
            address (str): Onchain address in 42-character hexadecimal format;
                           e.g. 0x0000000000000000000000000000000000000000.

        Returns:
            {
              type: "userDetails",
              txs: [
                {
                  time: int,
                  user: str,
                  action: {
                    type: str,
                    block: int,
                    hash: str
                    optional key(s) which depend on action type: *,
                  },
                  ...
                }
              ]
            }
        """
        return self.post("/explorer", {"type": "userDetails", "user": address})



from enum import Enum
from typing import Dict, Any, Union


class Interval(Enum):
    OneMinute = "1m"
    ThreeMinute = "3m"
    FiveMinute = "5m"
    FifteenMinute = "15m"
    ThirtyMinute = "30m"
    OneHour = "1h"
    TwoHour = "2h"
    FourHour = "4h"
    EightHour = "8h"
    TwelveHour = "12h"
    OneDay = "1d"
    ThreeDay = "3d"
    OneWeek = "1w"
    OneMonth = "1M"


Json = Union[Dict[str, Any], None]
